/**
 * @file Fonctions qui gerent l'autorisation des utilisateurs
 * @author ofacklam
 * @memberof GraphQL
 */

 import {Tools} from './tools'

/*
 * There are 7 levels of authorisation
 *  none : doesn't know the group / the user exists
 *  connectedOrOnplatal : knows the group exists, can use TOL
 *  viewer : can see the group
 *  member : part of the group
 *  speaker : allowed to speak for the group
 *  admin : admin of the group
 *  supervisor : allowed to take control of the group
 */

// These functions test if the given user has the right privileges for the given group

export class Authorisation {

    /**
     * @memberof GraphQL
     * @class Authorisation
     * @summary Autorisation des utilisateurs.
     * @classdesc Cette classe contient les fonctions d'autorisation des utilisateurs.
     */
    constructor() { }

    static PUBLICUSER = "public_user";
    static ONPLATALUSER = "public_onplatal_user";

    /**
     * @memberof GraphQL
     * @summary Fonction qui renvoit si l'utilisateur est connecté ou on-platal
     * @arg {string} uid - Identifiant de l'utilisateur.
     * @return {Promise(boolean)} Renvoie true si l'utilisateur est connecté ou on-platal
     * @static
     */
    static isConnectedOrOnplatal(uid: string): boolean {
        return (uid != this.PUBLICUSER);
    }
    
    /**
     * @memberof GraphQL
     * @summary Fonction qui renvoit si l'utilisateur est authentifié
     * @arg {string} uid - Identifiant de l'utilisateur.
     * @return {Promise(boolean)} Renvoie true si l'utilisateur est authentifié
     * @static
     */
    static isAuthenticated(uid: string): boolean {
        return(uid != this.PUBLICUSER && uid != this.ONPLATALUSER);
    }

    /**
     * @memberof GraphQL
     * @summary Fonction qui renvoit si l'utilisateur est viewer du groupe.
     * @arg {string} uid - Identifiant de l'utilisateur.
     * @arg {string} gid - Identifiant du groupe.
     * @return {Promise(boolean)} Renvoie true si l'utilisateur est viewer du groupe.
     * @static
     * @async
     */
    static async isViewer(uid: string, gid: string): Promise<boolean> {
        //ensure uid is valid !!!!!!!!
        if(Authorisation.isAuthenticated(uid)) {
            let groups = await Tools.viewerOf(uid);
            return (groups.simpleGroups.has(gid) || groups.metaGroups.has(gid));
        }
        return false;
    }

    /**
     * @memberof GraphQL
     * @summary Fonction qui renvoit si l'utilisateur est membre du groupe.
     * @arg {string} uid - Identifiant de l'utilisateur.
     * @arg {string} gid - Identifiant du groupe.
     * @return {Promise(boolean)} Renvoie true si l'utilisateur est membre du groupe.
     * @static
     * @async
     */
    static async isMember(uid: string, gid: string): Promise<boolean> {
        //ensure uid is valid !!!!!!!!
        if(Authorisation.isAuthenticated(uid)) {
            let groups = await Tools.memberOf(uid);
            return (groups.simpleGroups.has(gid) || groups.metaGroups.has(gid));
        }
        return false;
    }

    /**
     * @memberof GraphQL
     * @summary Fonction qui renvoit si l'utilisateur est speaker du groupe.
     * @arg {string} uid - Identifiant de l'utilisateur.
     * @arg {string} gid - Identifiant du groupe.
     * @return {Promise(boolean)} Renvoie true si l'utilisateur est speaker du groupe.
     * @static
     * @async
     */
    static async isSpeaker(uid: string, gid: string): Promise<boolean> {
        //ensure uid is valid !!!!!!!!
        if(Authorisation.isAuthenticated(uid)) {
            let groups = await Tools.speakerOf(uid);
            return (groups.simpleGroups.has(gid) || groups.metaGroups.has(gid));
        }
        return false;
    }

    /**
     * @memberof GraphQL
     * @summary Fonction qui renvoit si l'utilisateur est admin du groupe.
     * @arg {string} uid - Identifiant de l'utilisateur.
     * @arg {string} gid - Identifiant du groupe.
     * @return {Promise(boolean)} Renvoie true si l'utilisateur est admin du groupe.
     * @static
     * @async
     */
    static async isAdmin(uid: string, gid: string): Promise<boolean> {
        //ensure uid is valid !!!!!!!!
        if(Authorisation.isAuthenticated(uid)) {
            let groups = await Tools.adminOf(uid);
            return (groups.simpleGroups.has(gid) || groups.metaGroups.has(gid));
        }
        return false;
    }

    /**
     * @memberof GraphQL
     * @summary Fonction qui renvoit si l'utilisateur est supervisor du groupe.
     * @arg {string} uid - Identifiant de l'utilisateur.
     * @arg {string} gid - Identifiant du groupe.
     * @return {Promise(boolean)} Renvoie true si l'utilisateur est supervisor du groupe.
     * @static
     * @async
     */
    static async isSupervisor(uid: string, gid: string): Promise<boolean> {
        //ensure uid is valid !!!!!!!!
        if(Authorisation.isAuthenticated(uid)) {
            let groups = await Tools.supervisorOf(uid);
            return (groups.simpleGroups.has(gid) || groups.metaGroups.has(gid));
        }
        return false;
    }

}